# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_repr_chans.ipynb.

# %% auto 0
__all__ = ['chans']

# %% ../nbs/05_repr_chans.ipynb 5
from typing import Any, Optional as O

import torch
from matplotlib import pyplot as plt, axes, figure
from IPython.core.pylabtools import print_figure

from lovely_numpy.utils.utils import cached_property
from lovely_numpy.repr_chans import fig_chans
from lovely_numpy import config as np_config

from .utils.misc import to_numpy
from .utils.config import get_config


# %% ../nbs/05_repr_chans.ipynb 6
class ChanProxy():   
    def __init__(self, t: torch.Tensor):
        self.t = t
        self.params = dict(cmap = "twilight", 
                    cm_below="blue",
                    cm_above="red",
                    cm_ninf="cyan",
                    cm_pinf="fuchsia",
                    cm_nan="yellow",
                    view_width=966,
                    gutter_px=3,
                    frame_px=1,
                    scale=1,
                    cl=False,
                    ax=None)

    def __call__(self,
                 cmap       :O[str]=None, 
                 cm_below   :O[str]=None,
                 cm_above   :O[str]=None,
                 cm_ninf    :O[str]=None,
                 cm_pinf    :O[str]=None,
                 cm_nan     :O[str]=None,
                 view_width :O[int]=None,
                 gutter_px  :O[int]=None,
                 frame_px   :O[int]=None,
                 scale      :O[int]=None,
                 cl         :Any=None,
                 ax         :O[axes.Axes]=None):
        
        self.params.update( {   k:v for
                                k,v in locals().items()
                                if k != "self" and v is not None } )
        _ = self.fig # Trigger figure generation
        return self

    @cached_property
    def fig(self) -> figure.Figure:
        cfg = get_config()
        with np_config(fig_close=cfg.fig_close, fig_show=cfg.fig_show):
            return fig_chans(to_numpy(self.t), **self.params)

    def _repr_png_(self):
        return print_figure(self.fig, fmt="png", pad_inches=0,
            metadata={"Software": "Matplotlib, https://matplotlib.org/"})

# %% ../nbs/05_repr_chans.ipynb 7
def chans(  x: torch.Tensor,                # Input, shape=([...], H, W)
            cmap        :str    ="twilight",# Use matplotlib colormap by this name
            cm_below    :str    ="blue",    # Color for values below -1
            cm_above    :str    ="red",     # Color for values above 1
            cm_ninf     :str    ="cyan",    # Color for -inf values
            cm_pinf     :str    ="fuchsia", # Color for +inf values
            cm_nan      :str    ="yellow",  # Color for NaN values
            view_width  :int    =966,       # Try to produce an image at most this wide
            gutter_px   :int    =3,         # Draw write gutters when tiling the images
            frame_px    :int    =1,         # Draw black frame around each image
            scale       :int    =1,
            cl          :Any    =False,
            ax          :O[axes.Axes]=None
        ) -> ChanProxy:

    "Map tensor values to colors. RGB[A] color is added as channel-last"
    args = locals()
    del args["x"]

    return ChanProxy(x)(**args)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_repr_rgb.ipynb.

# %% auto 0
__all__ = ['rgb']

# %% ../nbs/01_repr_rgb.ipynb 5
from typing import Any, Optional as O

from matplotlib import axes, figure
from IPython.core.pylabtools import print_figure
from PIL import Image
import torch

from lovely_numpy.utils.utils import cached_property
from lovely_numpy.utils.pad import pad_frame_gutters
from lovely_numpy.utils.tile2d import hypertile
from lovely_numpy.repr_rgb import fig_rgb
from lovely_numpy import config as np_config

from .utils.misc import to_numpy
from .utils.config import get_config


# %% ../nbs/01_repr_rgb.ipynb 6
# This is here for the monkey-patched tensor use case.

# I want to be able to call both `tensor.rgb` and `tensor.rgb(stats)`. For the
# first case, the class defines `_repr_png_` to send the image to Jupyter. For
# the later case, it defines __call__, which accps the argument.

class RGBProxy():
    """Flexible `PIL.Image.Image` wrapper"""
    
    def __init__(self, t:torch.Tensor):
        assert t.ndim >= 3, f"Expecting at least 3 dimensions, got shape{t.shape}={t.dim()}"
        self.t =t
        self.params = dict(denorm   = None,
                            cl         = False,
                            gutter_px  = 3,     
                            frame_px   = 1,
                            scale      = 1,
                            view_width = 966,
                            ax = None)


    def __call__(self,
                denorm      :Any    =None,
                cl          :Any    =False,
                gutter_px   :O[int] =None,
                frame_px    :O[int] =None,
                scale       :O[int] =None,
                view_width  :O[int] =None,
                ax          :O[axes.Axes]=None):
        
        self.params.update( { k:v for
                            k,v in locals().items()
                            if k != "self" and v is not None } )
        _ = self.fig # Trigger figure generation
        return self

    @cached_property
    def fig(self) -> figure.Figure:
        cfg = get_config()
        with np_config(fig_close=cfg.fig_close, fig_show=cfg.fig_show):
            return fig_rgb(to_numpy(self.t), **self.params)

    def _repr_png_(self):
        return print_figure(self.fig, fmt="png", pad_inches=0,
            metadata={"Software": "Matplotlib, https://matplotlib.org/"})


# %% ../nbs/01_repr_rgb.ipynb 7
def rgb(x           :torch.Tensor,  # Tensor to display. [[...], C,H,W] or [[...], H,W,C]
        denorm      :Any =None,     # Reverse per-channel normalizatoin
        cl          :Any =False,    # Channel-last
        gutter_px   :int =3,        # If more than one tensor -> tile with this gutter width
        frame_px    :int =1,        # If more than one tensor -> tile with this frame width
        scale       :int =1,        # Scale up. Can't scale down.
        view_width  :int =966,      # target width of the image
        ax          :O[axes.Axes] =None # Use this Axes
        ) -> RGBProxy:

    args = locals()
    del args["x"]
    
    return RGBProxy(x)(**args)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/10_patch.ipynb.

# %% auto 0
__all__ = ['monkey_patch']

# %% ../nbs/10_patch.ipynb 3
from typing import Any, Optional as O

import torch
from fastcore.foundation import patch_to
from matplotlib import pyplot as plt, rcParams

from .repr_str import StrProxy
from .repr_rgb import RGBProxy
from .repr_plt import PlotProxy
from .repr_chans import ChanProxy

# %% ../nbs/10_patch.ipynb 4
def monkey_patch(cls=torch.Tensor):
    "Monkey-patch lovely features into `cls`" 

    if not hasattr(cls, '_plain_repr'):
        if cls is torch.Tensor:
            # This avoids invoking the _torch_function_ mechanism
            cls._plain_repr = torch._tensor_str._str
            cls._plain_str = torch._tensor_str._str
        else:
            cls._plain_repr = cls.__repr__
            cls._plain_str = cls.__str__

    @patch_to(cls)
    def __repr__(self: torch.Tensor, *, tensor_contents=None):        
        return str(StrProxy(self))

    # Plain - the old behavior
    @patch_to(cls, as_prop=True)
    def p(self: torch.Tensor, *, tensor_contents=None):
        return StrProxy(self, plain=True)

    # Verbose - print both stats and plain values
    @patch_to(cls, as_prop=True)
    def v(self: torch.Tensor, *, tensor_contents=None):
        return StrProxy(self, verbose=True)

    # .deeper and .deeper(...)
    @patch_to(cls, as_prop=True)
    def deeper(self: torch.Tensor):
        return StrProxy(self, depth=1)

    # .rgb and .rgb(...)
    @patch_to(cls, as_prop=True)
    def rgb(t: torch.Tensor):
        return RGBProxy(t)
    
    # .chans and .chans(...)
    @patch_to(cls, as_prop=True)
    def chans(t: torch.Tensor):
        return ChanProxy(t)

    # .plt and .plt(...)
    @patch_to(cls, as_prop=True)
    def plt(t: torch.Tensor):
        return PlotProxy(t)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_repr_plt.ipynb.

# %% auto 0
__all__ = ['plot']

# %% ../nbs/02_repr_plt.ipynb 4
import math
from typing import Union, Any, Optional as O

import torch
from matplotlib import pyplot as plt, axes, figure, rc_context, rcParams
from IPython.core.pylabtools import print_figure

from lovely_numpy.utils.utils import cached_property
from lovely_numpy.repr_plt import fig_plot
from lovely_numpy import config as np_config

from .repr_str import to_str, pretty_str
from .utils.misc import to_numpy
from .utils.config import get_config, config

# %% ../nbs/02_repr_plt.ipynb 5
# This is here for the monkey-patched tensor use case.
# Gives the ability to call both .plt and .plt(ax=ax).  

class PlotProxy(): 
    """Flexible `PIL.Image.Image` wrapper"""

    def __init__(self, x:torch.Tensor):
        self.x = x
        self.params = dict( center="zero",
                            max_s=10000,
                            plt0=True,
                            ax=None)

    def __call__(   self,
                    center  :O[str] =None,
                    max_s   :O[int] =None,
                    plt0    :Any    =None,
                    ax      :O[axes.Axes]=None):

        self.params.update( { k:v for
                    k,v in locals().items()
                    if k != "self" and v is not None } )
        
        _ = self.fig # Trigger figure generation
        return self

    @cached_property
    def fig(self) -> figure.Figure:
        cfg = get_config()

        with np_config( fig_close=cfg.fig_close,
                        fig_show=cfg.fig_show,
                        plt_seed=cfg.plt_seed ), config(show_mem_above=torch.inf):
            return fig_plot( to_numpy(self.x),
                            summary=to_str(self.x, color=False),
                            ddof=1,
                            **self.params)

    def _repr_png_(self):
        return print_figure(self.fig, fmt="png",
            metadata={"Software": "Matplotlib, https://matplotlib.org/"})

    def _repr_svg_(self):
        # Metadata and context for a mode deterministic svg generation
        metadata={
            "Date": None,
            "Creator": "Matplotlib, https://matplotlib.org/",
        }
        with rc_context({"svg.hashsalt": "1"}):
            svg_repr = print_figure(self.fig, fmt="svg", metadata=metadata)
        return svg_repr


# %% ../nbs/02_repr_plt.ipynb 6
def plot(   x       : torch.Tensor, # Tensor to explore
            center  :str    ="zero",    # Center plot on  `zero`, `mean`, or `range`
            max_s   :int    =10000,     # Draw up to this many samples. =0 to draw all
            plt0    :Any    =True,      # Take zero values into account
            ax      :O[axes.Axes]=None  # Optionally provide a matplotlib axes.
        ) -> PlotProxy:
    
    args = locals()
    del args["x"]

    return PlotProxy(x)(**args)




# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/03b_utils.misc.ipynb.

# %% auto 0
__all__ = ['to_numpy']

# %% ../../nbs/03b_utils.misc.ipynb 5
import numpy as np
import torch

# %% ../../nbs/03b_utils.misc.ipynb 6
def to_numpy(t: torch.Tensor) -> np.ndarray:
    t = t.detach().cpu()
    if t.dtype is torch.bfloat16: t = t.to(torch.float32)
    return t.numpy()



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_repr_str.ipynb.

# %% auto 0
__all__ = ['lovely']

# %% ../nbs/00_repr_str.ipynb 3
import warnings

import torch

from lovely_numpy import np_to_str_common, pretty_str, sparse_join, ansi_color, in_debugger, bytes_to_human
from lovely_numpy import config as lnp_config

from .utils.config import get_config, config
from .utils.misc import to_numpy

# %% ../nbs/00_repr_str.ipynb 6
def type_to_dtype(t: str) -> torch.dtype:
    "Convert str, e.g. 'float32' to torch.dtype e.g torch.float32"
    
    dtp = vars(torch)[t]
    assert isinstance(dtp, torch.dtype)
    return dtp

# %% ../nbs/00_repr_str.ipynb 8
dtnames = { type_to_dtype(k): v
                for k,v in {"float32": "",
                            "float16": "f16",
                            "float64": "f64",
                            "bfloat16": "bf16",
                            "uint8": "u8", # torch does not have uint16/32/64
                            "int8": "i8",
                            "int16": "i16",
                            "int32": "i32",
                            "int64": "i64",
                        }.items()
}

def short_dtype(x):
    # split(".") to torch.uint4 -> uint4
    return dtnames.get(x.dtype, str(x.dtype).split(".")[-1])

# %% ../nbs/00_repr_str.ipynb 10
def plain_repr(x: torch.Tensor):
    "Pick the right function to get a plain repr"
    return x._plain_repr() if hasattr(x, "_plain_repr") else repr(x)

def plain_str(x: torch.Tensor):
    "Pick the right function to get a plain str."
    return x._plain_str() if hasattr(x, "_plain_str") else str(x)

# %% ../nbs/00_repr_str.ipynb 11
def is_nasty(t: torch.Tensor):
    """Return true of any `t` values are inf or nan"""
    
    if t.numel() == 0: return False # amin/amax don't like zero-lenght tensors

    # Unlike .min()/.max(), amin/amax do not allocate extra GPU memory.

    t_min = t.amin().cpu()
    t_max = t.amax().cpu()

    return (t_min.isnan() or t_min.isinf() or t_max.isinf()).item()

# %% ../nbs/00_repr_str.ipynb 13
def torch_to_str_common(t: torch.Tensor,  # Input
                        color=True,       # ANSI color highlighting
                        ) -> str:
    
    if t.numel() == 0: return ansi_color("empty", "grey", color)

    # Unlike .min()/.max(), amin/amax do not allocate extra GPU memory.
    amin, amax = t.amin().cpu(), t.amax().cpu()

    zeros = ansi_color("all_zeros", "grey", color) if amin.eq(0) and amax.eq(0) and t.numel() > 1 else None
    # pinf = ansi_color("+Inf!", "red", color) if amax.isposinf() else None
    # ninf = ansi_color("-Inf!", "red", color) if amin.isneginf() else None
    # nan = ansi_color("NaN!", "red", color) if amin.isnan() else None

    # attention = sparse_join([zeros,pinf,ninf,nan])
    # numel = f"n={t.numel()}" if t.numel() > 5 and max(t.shape) != t.numel() else None

    summary = None
    if not zeros:
        minmax = f"x∈[{pretty_str(amin)}, {pretty_str(amax)}]" if t.numel() > 2 else None
        meanstd = f"μ={pretty_str(t.mean())} σ={pretty_str(t.std())}" if t.numel() >= 2 else None
        summary = sparse_join([minmax, meanstd])

    return sparse_join([ summary, zeros ])

# %% ../nbs/00_repr_str.ipynb 14
# tensor.is_cpu was only introduced in 1.13.
def is_cpu(t: torch.Tensor):
    return t.device == torch.device("cpu")

# %% ../nbs/00_repr_str.ipynb 15
@torch.no_grad()
def to_str(t: torch.Tensor,
            plain: bool=False,
            verbose: bool=False,
            depth=0,
            lvl=0,
            color=None) -> str:

    if plain:
        return plain_repr(t)

    conf = get_config()

    tname = "tensor" if type(t) is torch.Tensor else type(t).__name__.split(".")[-1]
    shape = str(list(t.shape)) if t.ndim else None
    type_str = sparse_join([tname, shape], sep="")
    
    dev = str(t.device) if t.device.type != "cpu" else None
    dtype = short_dtype(t)
    grad_fn = t.grad_fn.name() if t.grad_fn else None
    # PyTorch does not want you to know, but all `grad_fn``
    # tensors actuall have `requires_grad=True`` too.
    grad = "grad" if t.requires_grad else None 
    

    # For complex tensors, just show the shape / size part for now.
    if not t.is_complex():
        if color is None: color=conf.color
        if in_debugger(): color=False
        # `lovely-numpy` is used to calculate stats when doing so on GPU would require
        # memory allocation (not float tensors, tensors with bad numbers), or if the
        # data is on CPU (because numpy is faster).
        #
        # Temporarily set the numpy config to match our config for consistency.
        with lnp_config(precision=conf.precision,
                        threshold_min=conf.threshold_min,
                        threshold_max=conf.threshold_max,
                        sci_mode=conf.sci_mode):
            if is_nasty(t) or not t.is_floating_point():
                common = np_to_str_common(to_numpy(t), color=color, ddof=1)
            else:
                common = torch_to_str_common(t, color=color)

            numel = None
            nbytes = t.numel() * t.element_size()
            if t.shape and max(t.shape) != t.numel():
                numel = f"n={t.numel()}"
                if get_config().show_mem_above <= nbytes:
                    numel = sparse_join([numel, f"({bytes_to_human(nbytes)})"])
            elif get_config().show_mem_above <= nbytes:
                numel = bytes_to_human(nbytes)

            vals = pretty_str(to_numpy(t)) if 0 < t.numel() <= 10 else None
            res = sparse_join([type_str, dtype, numel, common, grad, grad_fn, dev, vals])
    else:
        res = plain_repr(t)


    if verbose:
        res += "\n" + plain_repr(t)

    if depth and t.dim() > 1:
        with config(show_mem_above=torch.inf):
            deep_width = min((t.shape[0]), conf.deeper_width) # Print at most this many lines
            deep_lines = [ " "*conf.indent*(lvl+1) + to_str(t[i,:], depth=depth-1, lvl=lvl+1)
                                for i in range(deep_width)] 

            # If we were limited by width, print ...
            if deep_width < t.shape[0]: deep_lines.append(" "*conf.indent*(lvl+1) + "...")

            res += "\n" + "\n".join(deep_lines)

    return res

# %% ../nbs/00_repr_str.ipynb 16
def history_warning():
    "Issue a warning (once) ifw e are running in IPYthon with output cache enabled"

    if "get_ipython" in globals() and get_ipython().cache_size > 0:
        warnings.warn("IPYthon has its output cache enabled. See https://xl0.github.io/lovely-tensors/history.html")

# %% ../nbs/00_repr_str.ipynb 19
class StrProxy():
    def __init__(self, t: torch.Tensor, plain=False, verbose=False, depth=0, lvl=0, color=None):
        self.t = t
        self.plain = plain
        self.verbose = verbose
        self.depth=depth
        self.lvl=lvl
        self.color=color
        history_warning()
    
    def __repr__(self):
        return to_str(self.t, plain=self.plain, verbose=self.verbose,
                      depth=self.depth, lvl=self.lvl, color=self.color)

    # This is used for .deeper attribute and .deeper(depth=...).
    # The second onthe results in a __call__.
    def __call__(self, depth=1):
        return StrProxy(self.t, depth=depth)

# %% ../nbs/00_repr_str.ipynb 20
def lovely(t: torch.Tensor, # Tensor of interest
            verbose=False,  # Whether to show the full tensor
            plain=False,    # Just print if exactly as before
            depth=0,        # Show stats in depth
            color=None):    # Force color (True/False) or auto.
    return StrProxy(t, verbose=verbose, plain=plain, depth=depth, color=color)



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/lovely-tensors',
                'doc_host': 'https://xl0.github.io',
                'git_url': 'https://github.com/xl0/lovely-tensors',
                'lib_path': 'lovely_tensors'},
  'syms': { 'lovely_tensors.patch': {'lovely_tensors.patch.monkey_patch': ('patch.html#monkey_patch', 'lovely_tensors/patch.py')},
            'lovely_tensors.repr_chans': { 'lovely_tensors.repr_chans.ChanProxy': ( 'repr_chans.html#chanproxy',
                                                                                    'lovely_tensors/repr_chans.py'),
                                           'lovely_tensors.repr_chans.ChanProxy.__call__': ( 'repr_chans.html#chanproxy.__call__',
                                                                                             'lovely_tensors/repr_chans.py'),
                                           'lovely_tensors.repr_chans.ChanProxy.__init__': ( 'repr_chans.html#chanproxy.__init__',
                                                                                             'lovely_tensors/repr_chans.py'),
                                           'lovely_tensors.repr_chans.ChanProxy._repr_png_': ( 'repr_chans.html#chanproxy._repr_png_',
                                                                                               'lovely_tensors/repr_chans.py'),
                                           'lovely_tensors.repr_chans.ChanProxy.fig': ( 'repr_chans.html#chanproxy.fig',
                                                                                        'lovely_tensors/repr_chans.py'),
                                           'lovely_tensors.repr_chans.chans': ('repr_chans.html#chans', 'lovely_tensors/repr_chans.py')},
            'lovely_tensors.repr_plt': { 'lovely_tensors.repr_plt.PlotProxy': ('repr_plt.html#plotproxy', 'lovely_tensors/repr_plt.py'),
                                         'lovely_tensors.repr_plt.PlotProxy.__call__': ( 'repr_plt.html#plotproxy.__call__',
                                                                                         'lovely_tensors/repr_plt.py'),
                                         'lovely_tensors.repr_plt.PlotProxy.__init__': ( 'repr_plt.html#plotproxy.__init__',
                                                                                         'lovely_tensors/repr_plt.py'),
                                         'lovely_tensors.repr_plt.PlotProxy._repr_png_': ( 'repr_plt.html#plotproxy._repr_png_',
                                                                                           'lovely_tensors/repr_plt.py'),
                                         'lovely_tensors.repr_plt.PlotProxy._repr_svg_': ( 'repr_plt.html#plotproxy._repr_svg_',
                                                                                           'lovely_tensors/repr_plt.py'),
                                         'lovely_tensors.repr_plt.PlotProxy.fig': ( 'repr_plt.html#plotproxy.fig',
                                                                                    'lovely_tensors/repr_plt.py'),
                                         'lovely_tensors.repr_plt.plot': ('repr_plt.html#plot', 'lovely_tensors/repr_plt.py')},
            'lovely_tensors.repr_rgb': { 'lovely_tensors.repr_rgb.RGBProxy': ('repr_rgb.html#rgbproxy', 'lovely_tensors/repr_rgb.py'),
                                         'lovely_tensors.repr_rgb.RGBProxy.__call__': ( 'repr_rgb.html#rgbproxy.__call__',
                                                                                        'lovely_tensors/repr_rgb.py'),
                                         'lovely_tensors.repr_rgb.RGBProxy.__init__': ( 'repr_rgb.html#rgbproxy.__init__',
                                                                                        'lovely_tensors/repr_rgb.py'),
                                         'lovely_tensors.repr_rgb.RGBProxy._repr_png_': ( 'repr_rgb.html#rgbproxy._repr_png_',
                                                                                          'lovely_tensors/repr_rgb.py'),
                                         'lovely_tensors.repr_rgb.RGBProxy.fig': ( 'repr_rgb.html#rgbproxy.fig',
                                                                                   'lovely_tensors/repr_rgb.py'),
                                         'lovely_tensors.repr_rgb.rgb': ('repr_rgb.html#rgb', 'lovely_tensors/repr_rgb.py')},
            'lovely_tensors.repr_str': { 'lovely_tensors.repr_str.StrProxy': ('repr_str.html#strproxy', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.StrProxy.__call__': ( 'repr_str.html#strproxy.__call__',
                                                                                        'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.StrProxy.__init__': ( 'repr_str.html#strproxy.__init__',
                                                                                        'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.StrProxy.__repr__': ( 'repr_str.html#strproxy.__repr__',
                                                                                        'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.history_warning': ( 'repr_str.html#history_warning',
                                                                                      'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.is_cpu': ('repr_str.html#is_cpu', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.is_nasty': ('repr_str.html#is_nasty', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.lovely': ('repr_str.html#lovely', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.plain_repr': ('repr_str.html#plain_repr', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.plain_str': ('repr_str.html#plain_str', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.short_dtype': ('repr_str.html#short_dtype', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.to_str': ('repr_str.html#to_str', 'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.torch_to_str_common': ( 'repr_str.html#torch_to_str_common',
                                                                                          'lovely_tensors/repr_str.py'),
                                         'lovely_tensors.repr_str.type_to_dtype': ( 'repr_str.html#type_to_dtype',
                                                                                    'lovely_tensors/repr_str.py')},
            'lovely_tensors.utils.config': { 'lovely_tensors.utils.config.Config': ( 'utils.config.html#config',
                                                                                     'lovely_tensors/utils/config.py'),
                                             'lovely_tensors.utils.config.Config.__init__': ( 'utils.config.html#config.__init__',
                                                                                              'lovely_tensors/utils/config.py'),
                                             'lovely_tensors.utils.config._Default': ( 'utils.config.html#_default',
                                                                                       'lovely_tensors/utils/config.py'),
                                             'lovely_tensors.utils.config._Default.__repr__': ( 'utils.config.html#_default.__repr__',
                                                                                                'lovely_tensors/utils/config.py'),
                                             'lovely_tensors.utils.config.config': ( 'utils.config.html#config',
                                                                                     'lovely_tensors/utils/config.py'),
                                             'lovely_tensors.utils.config.get_config': ( 'utils.config.html#get_config',
                                                                                         'lovely_tensors/utils/config.py'),
                                             'lovely_tensors.utils.config.set_config': ( 'utils.config.html#set_config',
                                                                                         'lovely_tensors/utils/config.py')},
            'lovely_tensors.utils.misc': { 'lovely_tensors.utils.misc.to_numpy': ( 'utils.misc.html#to_numpy',
                                                                                   'lovely_tensors/utils/misc.py')}}}
